package org.robokind.headless.motion;

import java.io.File;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Logger;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.robokind.api.common.lifecycle.ManagedService;
import org.robokind.api.common.lifecycle.ServiceLifecycleProvider;
import org.robokind.api.common.lifecycle.utils.SimpleLifecycle;
import org.robokind.api.common.osgi.lifecycle.OSGiComponent;
import org.robokind.api.common.osgi.lifecycle.ConfiguredServiceLifecycle;
import org.robokind.api.common.osgi.lifecycle.ConfiguredServiceParams;
import org.robokind.api.common.osgi.lifecycle.OSGiComponentFactory;
import org.robokind.api.common.services.Constants;
import org.robokind.api.motion.Robot;
import org.robokind.api.motion.lifecycle.RobotJointGroupLifecycle;
import org.robokind.api.motion.servos.ServoRobot;
import org.robokind.api.motion.utils.RobotUtils;
import org.robokind.extern.utils.apache_commons_configuration.ConfigUtils;
import org.robokind.impl.messaging.config.RKMessagingConfigUtils;
import org.robokind.impl.motion.config.RobotConfigXMLReader;
import org.robokind.impl.motion.jointgroup.RobotJointGroupConfigXMLReader;
import org.robokind.impl.motion.lifecycle.RemoteRobotHostServiceGroup;

public class Activator implements BundleActivator {
    private final static Logger theLogger = 
            Logger.getLogger(Activator.class.getName());
    
    private final static String ROBOT_CONFIG_FILE = "./resources/robot.xml";
    private final static Robot.Id ROBOT_ID = new Robot.Id("myRobot");
    
    @Override
    public void start(BundleContext context) throws Exception {
        theLogger.info("Connecting to Servo Robot.");
        startRobot(context, ROBOT_CONFIG_FILE);
        theLogger.info("Connection Successful.");
        String connectionConfigId = "remoteRobotConnectionConfig";
        RKMessagingConfigUtils.registerConnectionConfig(
                connectionConfigId, "127.0.0.1", 
                null, new OSGiComponentFactory(context));
        new RemoteRobotHostServiceGroup(
                context, ROBOT_ID, "host", "client", 
                connectionConfigId, null).start();
        RobotUtils.startDefaultBlender(
                context, ROBOT_ID, theDefaultBlenderInterval);
        loadJointGroup(ROBOT_ID, context, "./resources/jointgroup.xml");
    }
    
    private final static long theDefaultBlenderInterval = 40L;
    
    private static void startRobot(BundleContext context, String path){
        theLogger.info("Opening Robot Config File.");
        File file = ConfigUtils.getFileSystemAdapter().openFile(path);
        if(file == null){
            return;
        }
        ConfiguredServiceParams<Robot,Object,File> robotParams = 
                new ConfiguredServiceParams(
                        Robot.class, null, File.class, null, file, null, 
                        ServoRobot.VERSION, RobotConfigXMLReader.VERSION);
        Properties props = new Properties();
        props.put(Robot.PROP_ID, ROBOT_ID);
        ConfiguredServiceLifecycle<Robot,?,File> robotLifecycle = 
                new ConfiguredServiceLifecycle<Robot,Object,File>(robotParams){
                    @Override
                    protected Robot create(Map dependencies){
                        Robot robot = super.create(dependencies);
                        if(robot == null){
                            return null;
                        }
                        robot.connect();
                        return robot;
                    }
                };
        OSGiComponent robotComp = 
                new OSGiComponent(
                        context, robotLifecycle, null, props, true);
        robotComp.start();
    }
    
    protected static ManagedService loadJointGroup(
            Robot.Id robotId, BundleContext context, String path){
        File file = ConfigUtils.getFileSystemAdapter().openFile(path);
        if(file == null){
            return null;
        }
        String paramId = "robot/" + robotId + "/jointgroup/config/param/xml";
        launchJointGroupConfig(context, file, paramId);
        return launchJointGroupLifecycle(context, robotId, paramId);
    }
    
    protected static OSGiComponent launchJointGroupLifecycle(
            BundleContext context, Robot.Id robotId, String paramId){
        RobotJointGroupLifecycle<File> lifecycle =
                new RobotJointGroupLifecycle<File>(robotId, File.class, 
                        paramId, RobotJointGroupConfigXMLReader.VERSION);
        OSGiComponent jointGroupComp = new OSGiComponent(context, lifecycle);
        jointGroupComp.start();
        return null;
    }
    
    protected static OSGiComponent launchJointGroupConfig(BundleContext context, 
            File jointGroupConfigXML, String configFileId){
        Properties props = new Properties();
        props.put(Constants.CONFIG_PARAM_ID, configFileId);
        props.put(Constants.CONFIG_FORMAT_VERSION, 
                RobotJointGroupConfigXMLReader.VERSION.toString());
        ServiceLifecycleProvider lifecycle = new SimpleLifecycle(
                        jointGroupConfigXML, File.class, props);
        OSGiComponent paramComp = new OSGiComponent(context, lifecycle);
        paramComp.start();
        return paramComp;
    }

    @Override
    public void stop(BundleContext context) throws Exception {
        // TODO add deactivation code here
    }

}